@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2023
+# © Copyright EnterpriseDB UK Limited 2011-2025
 #
 # This file is part of Barman.
 #
@@ -160,20 +160,26 @@ class BackupExecutor(with_metaclass(ABCMeta, RemoteStatusMixin)):
 
     def _purge_unused_wal_files(self, backup_info):
         """
-        It the provided backup is the first, purge all WAL files before the
-        backup start.
+        If the provided backup is the first, purge unused WAL files before the backup
+        start.
 
-        :param barman.infofile.LocalBackupInfo backup_info: the backup to check
+
+        .. note::
+            If ``worm_mode`` is enabled, then we don't remove those WAL files
+            because they are (should be) stored in an immutable storage, and at
+            this point the grace period might already have been expired.
+
+        :param barman.infofile.LocalBackupInfo backup_info: The backup to check.
         """
-
-        # Do nothing if the begin_wal is not defined yet
         if backup_info.begin_wal is None:
             return
 
-        # If this is the first backup, purge unused WAL files
         previous_backup = self.backup_manager.get_previous_backup(backup_info.backup_id)
         if not previous_backup:
             output.info("This is the first backup for server %s", self.config.name)
+            if self.config.worm_mode is True:
+                output.info("'worm_mode' is enabled, skip purging of unused WAL files.")
+                return
             removed = self.backup_manager.remove_wal_before_backup(backup_info)
             if removed:
                 # report the list of the removed WAL files
@@ -1672,7 +1678,7 @@ class SnapshotBackupExecutor(ExternalBackupExecutor):
             to be backed up are attached.
         :param list[str] snapshot_disks: A list containing the names of the disks for
             which snapshots should be taken at backup time.
-        :rtype tuple[list[str],list[str]]
+        :rtype: tuple[list[str],list[str]]
         :return: A tuple where the first element is a list of all disks which are not
             attached to the VM instance and the second element is a list of all disks
             which are attached but not mounted.
@@ -1694,7 +1700,7 @@ class SnapshotBackupExecutor(ExternalBackupExecutor):
                 # Ignore disks which were not attached
                 continue
             except SnapshotBackupException as exc:
-                logging.warn("Error resolving mount point: {}".format(exc))
+                _logger.warning("Error resolving mount point: {}".format(exc))
                 mount_point = None
             if mount_point is None:
                 unmounted_disks.append(disk)
@@ -1894,10 +1900,9 @@ class BackupStrategy(with_metaclass(ABCMeta, object)):
         Fill a backup info with information from a start_backup
 
         :param barman.infofile.BackupInfo backup_info: object
-        representing a
-            backup
+            representing a backup
         :param DictCursor start_info: the result of the pg_backup_start
-        command
+            command
         """
         backup_info.set_attribute("status", BackupInfo.STARTED)
         backup_info.set_attribute("begin_time", start_info["timestamp"])
