@@ -35,7 +35,12 @@ import barman.utils
 from barman import output
 from barman.annotations import KeepManager
 from barman.backup_manifest import BackupManifest
-from barman.config import ConfigChangesProcessor, RecoveryOptions, parse_staging_path
+from barman.config import (
+    ConfigChangesProcessor,
+    RecoveryOptions,
+    parse_combine_mode,
+    parse_staging_path,
+)
 from barman.exceptions import (
     BadXlogSegmentName,
     LockFileBusy,
@@ -632,12 +637,19 @@ def list_backups(args):
     """
     List available backups for the given server (supports 'all')
     """
-    servers = get_server_list(args, skip_inactive=True)
+    servers = get_server_list(args, skip_inactive=False, skip_disabled=False)
     for name in sorted(servers):
         server = servers[name]
 
         # Skip the server (apply general rule)
-        if not manage_server_command(server, name):
+        if not manage_server_command(
+            server,
+            name,
+            skip_inactive=False,
+            skip_disabled=False,
+            inactive_is_error=False,
+            disabled_is_error=False,
+        ):
             continue
 
         output.init("list_backup", name, minimal=args.minimal)
@@ -660,12 +672,19 @@ def status(args):
     """
     Shows live information and status of the PostgreSQL server
     """
-    servers = get_server_list(args, skip_inactive=True)
+    servers = get_server_list(args, skip_inactive=False, skip_disabled=False)
     for name in sorted(servers):
         server = servers[name]
 
         # Skip the server (apply general rule)
-        if not manage_server_command(server, name):
+        if not manage_server_command(
+            server,
+            name,
+            skip_inactive=False,
+            skip_disabled=False,
+            inactive_is_error=False,
+            disabled_is_error=False,
+        ):
             continue
 
         output.init("status", name)
@@ -942,6 +961,16 @@ def rebuild_xlogdb(args):
                 "values are `local` and `remote`."
             ),
         ),
+        argument(
+            "--combine-mode",
+            help=(
+                "Specifies a copy mode for `pg_combinebackup` when combining "
+                "incremental backups during a restore."
+            ),
+            type=parse_combine_mode,
+            dest="combine_mode",
+            choices=["copy", "link", "clone", "copy-file-range"],
+        ),
         argument(
             "--recovery-conf-filename",
             dest="recovery_conf_filename",
@@ -950,6 +979,28 @@ def rebuild_xlogdb(args):
                 "added for PostgreSQL 12 and later (default: postgresql.auto.conf)."
             ),
         ),
+        argument(
+            "--recovery-option-port",
+            dest="recovery_option_port",
+            help=(
+                "Port number to use in the restore command when invoking "
+                "barman-wal-restore."
+            ),
+        ),
+        argument(
+            "--delta-restore",
+            dest="delta_restore",
+            help="Enable delta restore mode.",
+            action="store_true",
+            default=SUPPRESS,
+        ),
+        argument(
+            "--no-delta-restore",
+            help="Disable delta restore mode.",
+            dest="delta_restore",
+            action="store_false",
+            default=SUPPRESS,
+        ),
         argument(
             "--snapshot-recovery-instance",
             help="Instance where the disks recovered from the snapshots are attached",
@@ -982,7 +1033,13 @@ def restore(args):
     """
     Restore a server at a given time, name, LSN or xid
     """
-    server = get_server(args)
+    server = get_server(
+        args,
+        skip_inactive=False,
+        skip_disabled=False,
+        inactive_is_error=False,
+        disabled_is_error=True,
+    )
 
     # PostgreSQL supports multiple parameters to specify when the recovery
     # process will end, and in that case the last entry in recovery
@@ -1234,6 +1291,8 @@ def restore(args):
         server.config.parallel_jobs_start_batch_period = args.jobs_start_batch_period
     if hasattr(args, "bwlimit"):
         server.config.bandwidth_limit = args.bwlimit
+    if args.combine_mode is not None:
+        server.config.combine_mode = args.combine_mode
 
     if hasattr(args, "network_compression"):
         if args.network_compression and args.remote_ssh_command is None:
@@ -1246,6 +1305,36 @@ def restore(args):
             output.close_and_exit()
         server.config.network_compression = args.network_compression
 
+    if hasattr(args, "delta_restore"):
+        if args.delta_restore:
+            server.config.recovery_options.add(RecoveryOptions.DELTA_RESTORE)
+        elif RecoveryOptions.DELTA_RESTORE in server.config.recovery_options:
+            server.config.recovery_options.remove(RecoveryOptions.DELTA_RESTORE)
+
+    delta_restore = RecoveryOptions.DELTA_RESTORE in server.config.recovery_options
+    if delta_restore:
+        # Local restore does not support delta for non-plain backups.
+        if not args.remote_ssh_command:
+            if any(
+                [
+                    backup_info.is_incremental,
+                    backup_info.compression,
+                    backup_info.encryption,
+                ]
+            ):
+                output.error(
+                    "Cannot restore a backup locally with delta mode when the backup is "
+                    "not plain."
+                )
+                output.close_and_exit()
+        # Remote restore does not support delta when staging_location is remote.
+        elif server.config.staging_location == "remote":
+            output.error(
+                "Cannot restore a backup remotely with delta mode when "
+                "'staging_location=remote'."
+            )
+            output.close_and_exit()
+
     if backup_info.snapshots_info is not None:
         missing_args = []
         if not args.snapshot_recovery_instance:
@@ -1315,6 +1404,7 @@ def restore(args):
                 target_action=getattr(args, "target_action", None),
                 standby_mode=getattr(args, "standby_mode", None),
                 recovery_conf_filename=args.recovery_conf_filename,
+                recovery_option_port=args.recovery_option_port,
                 **snapshot_kwargs,
             )
         except RecoveryException as exc:
@@ -1604,7 +1694,13 @@ def show_backup(args):
     """
     This method shows a single backup information
     """
-    server = get_server(args)
+    server = get_server(
+        args,
+        skip_inactive=False,
+        skip_disabled=False,
+        inactive_is_error=False,
+        disabled_is_error=False,
+    )
 
     # Retrieves the backup
     backup_info = parse_backup_id(server, args)
@@ -1646,7 +1742,13 @@ def list_files(args):
     """
     List all the files for a single backup
     """
-    server = get_server(args)
+    server = get_server(
+        args,
+        skip_inactive=False,
+        skip_disabled=False,
+        inactive_is_error=False,
+        disabled_is_error=False,
+    )
 
     # Retrieves the backup
     backup_info = parse_backup_id(server, args)
@@ -1682,7 +1784,13 @@ def delete(args):
     """
     Delete a backup
     """
-    server = get_server(args)
+    server = get_server(
+        args,
+        skip_inactive=False,
+        skip_disabled=False,
+        inactive_is_error=False,
+        disabled_is_error=True,
+    )
 
     # Retrieves the backup
     backup_id = parse_backup_id(server, args)
@@ -1769,7 +1877,13 @@ def get_wal(args):
     The content will be streamed on standard output unless
     the --output-directory option is specified.
     """
-    server = get_server(args, inactive_is_error=True)
+    server = get_server(
+        args,
+        skip_inactive=False,
+        skip_disabled=False,
+        inactive_is_error=False,
+        disabled_is_error=True,
+    )
 
     if getattr(args, "test", None):
         output.info(
@@ -1976,7 +2090,13 @@ def verify_backup(args):
     verify a backup for the given server and backup id
     """
     # get barman.server.Server
-    server = get_server(args)
+    server = get_server(
+        args,
+        skip_inactive=False,
+        skip_disabled=False,
+        inactive_is_error=False,
+        disabled_is_error=False,
+    )
     # Raises an error if wrong backup
     backup_info = parse_backup_id(server, args)
     # get backup path
@@ -2125,7 +2245,7 @@ def check_wal_archive(args):
                 server.config.name,
                 force_str(err),
             )
-            logging.error(msg)
+            _logger.error(msg)
             output.error(msg)
             output.close_and_exit()
 
@@ -2162,7 +2282,13 @@ def config_switch(args):
         output.error("Either a model name or '--reset' flag need to be given")
         return
 
-    server = get_server(args, skip_inactive=False)
+    server = get_server(
+        args,
+        skip_inactive=False,
+        skip_disabled=False,
+        inactive_is_error=False,
+        disabled_is_error=False,
+    )
 
     if server is not None:
         if args.reset:
