@@ -24,7 +24,6 @@ from contextlib import closing
 from barman.clients.cloud_cli import (
     CLIErrorExit,
     GeneralErrorExit,
-    NetworkErrorExit,
     UrlArgumentType,
     add_tag_argument,
     create_argument_parser,
@@ -37,6 +36,8 @@ from barman.exceptions import BarmanException
 from barman.utils import check_positive, check_size, check_tag, force_str
 from barman.xlog import hash_dir, is_any_xlog_file, is_history_file
 
+_logger = logging.getLogger(__name__)
+
 
 def __is_hook_script():
     """Check the environment and determine if we are running as a hook script"""
@@ -76,13 +77,17 @@ def main(args=None):
 
     # Validate the WAL file name before uploading it
     if not is_any_xlog_file(config.wal_path):
-        logging.error("%s is an invalid name for a WAL file" % config.wal_path)
+        _logger.error("%s is an invalid name for a WAL file" % config.wal_path)
         raise CLIErrorExit()
 
     try:
         cloud_interface = get_cloud_interface(config)
 
         with closing(cloud_interface):
+            if config.test:
+                cloud_interface.verify_cloud_connectivity_and_bucket_existence()
+                raise SystemExit(0)
+
             uploader = CloudWalUploader(
                 cloud_interface=cloud_interface,
                 server_name=config.server_name,
@@ -90,15 +95,6 @@ def main(args=None):
                 compression_level=config.compression_level,
             )
 
-            if not cloud_interface.test_connectivity():
-                raise NetworkErrorExit()
-            # If test is requested, just exit after connectivity test
-            elif config.test:
-                raise SystemExit(0)
-
-            # TODO: Should the setup be optional?
-            cloud_interface.setup_bucket()
-
             upload_kwargs = {}
             if is_history_file(config.wal_path):
                 upload_kwargs["override_tags"] = config.history_tags
@@ -106,8 +102,8 @@ def main(args=None):
             uploader.upload_wal(config.wal_path, **upload_kwargs)
 
     except Exception as exc:
-        logging.error("Barman cloud WAL archiver exception: %s", force_str(exc))
-        logging.debug("Exception details:", exc_info=exc)
+        _logger.error("Barman cloud WAL archiver exception: %s", force_str(exc))
+        _logger.debug("Exception details:", exc_info=exc)
         raise GeneralErrorExit()
 
 
