@@ -93,6 +93,8 @@ ENCRYPTION_VALUES = ["none", "gpg"]
 # Staging location options. Indicates whether staging_path is a local or remote path
 STAGING_LOCATIONS = ["local", "remote"]
 
+COMBINE_MODES = ["copy", "link", "clone", "copy-file-range"]
+
 
 class CsvOption(set):
     """
@@ -195,9 +197,10 @@ class RecoveryOptions(CsvOption):
 
     # constants containing labels for allowed values
     GET_WAL = "get-wal"
+    DELTA_RESTORE = "delta-restore"
 
     # list holding all the allowed values for the RecoveryOptions class
-    value_list = [GET_WAL]
+    value_list = [GET_WAL, DELTA_RESTORE]
 
 
 def parse_boolean(value):
@@ -531,6 +534,22 @@ def parse_create_slot(value):
     )
 
 
+def parse_combine_mode(value):
+    """
+    Parse a string to a valid ``combine_mode`` value.
+
+    Valid values are defined in :data:`COMBINE_MODES`.
+    """
+    if value is None:
+        return None
+    value = value.lower()
+    if value not in COMBINE_MODES:
+        raise ValueError(
+            "Invalid value: %s (options are: %s)" % (value, ", ".join(COMBINE_MODES))
+        )
+    return value
+
+
 class BaseConfig(object):
     """
     Contains basic methods for handling configuration of Servers and Models.
@@ -625,9 +644,9 @@ class ServerConfig(BaseConfig):
         "compression",
         "compression_level",
         "conninfo",
-        "custom_compression_filter",
-        "custom_decompression_filter",
-        "custom_compression_magic",
+        "custom_compression_filter",  # Deprecated in 3.17
+        "custom_decompression_filter",  # Deprecated in 3.17
+        "custom_compression_magic",  # Deprecated in 3.17
         "description",
         "disabled",
         "encryption",
@@ -643,6 +662,7 @@ class ServerConfig(BaseConfig):
         "last_backup_maximum_age",
         "last_backup_minimum_size",
         "last_wal_maximum_age",
+        "combine_mode",
         "local_staging_path",  # Deprecated, replaced by staging_path and staging_location
         "max_incoming_wals_queue",
         "minimum_redundancy",
@@ -733,9 +753,9 @@ class ServerConfig(BaseConfig):
         "compression_level",
         "configuration_files_directory",
         "create_slot",
-        "custom_compression_filter",
-        "custom_decompression_filter",
-        "custom_compression_magic",
+        "custom_compression_filter",  # Deprecated in 3.17
+        "custom_decompression_filter",  # Deprecated in 3.17
+        "custom_compression_magic",  # Deprecated in 3.17
         "encryption",
         "encryption_key_id",
         "encryption_passphrase_command",
@@ -746,6 +766,7 @@ class ServerConfig(BaseConfig):
         "last_backup_maximum_age",
         "last_backup_minimum_size",
         "last_wal_maximum_age",
+        "combine_mode",
         "local_staging_path",
         "max_incoming_wals_queue",
         "minimum_redundancy",
@@ -817,6 +838,7 @@ class ServerConfig(BaseConfig):
         "immediate_checkpoint": "false",
         "incoming_wals_directory": "%(backup_directory)s/incoming",
         "keepalive_interval": "60",
+        "combine_mode": "copy",
         "minimum_redundancy": "0",
         "network_compression": "false",
         "parallel_jobs": "1",
@@ -877,6 +899,7 @@ class ServerConfig(BaseConfig):
         "last_backup_maximum_age": parse_time_interval,
         "last_backup_minimum_size": parse_si_suffix,
         "last_wal_maximum_age": parse_time_interval,
+        "combine_mode": parse_combine_mode,
         "local_staging_path": parse_staging_path,
         "max_incoming_wals_queue": int,
         "network_compression": parse_boolean,
@@ -1023,16 +1046,16 @@ class ServerConfig(BaseConfig):
               * ``source``: the file which provides the effective value, if
                 the option has been configured by the user, otherwise ``None``.
         """
-        json_dict = dict(vars(self))
-
         # remove references that should not go inside the
-        # `servers -> SERVER -> config` key in the barman diagnose output
-        # ideally we should change this later so we only consider configuration
-        # options, as things like `msg_list` are going to the `config` key,
-        # i.e. we might be interested in considering only `ServerConfig.KEYS`
-        # here instead of `vars(self)`
-        for key in ["config", "_active_model_file", "active_model"]:
-            del json_dict[key]
+        # `servers -> SERVER -> config` key in the barman diagnose output.
+        # Consider only configuration options, from `ServerConfig.KEYS`. Things like
+        # `msg_list` is going to be a sibling of `config` key.
+        json_dict = {
+            key: getattr(self, key)
+            for key in self.KEYS
+            if key not in {"config", "_active_model_file", "active_model"}
+        }
+        json_dict["name"] = self.name
 
         # options that are override by the model
         override_options = set()
@@ -1089,7 +1112,7 @@ class ServerConfig(BaseConfig):
         self.msg_list.extend(msg_list)
         self.disabled = True
 
-    def get_wal_conninfo(self):
+    def get_wal_conninfo(self, check_strategy=False):
         """
         Return WAL-specific conninfo strings for this server.
 
@@ -1102,17 +1125,31 @@ class ServerConfig(BaseConfig):
         :return: Tuple consisting of the ``wal_streaming_conninfo`` and
             ``wal_conninfo``.
         """
-        # If `wal_streaming_conninfo` is not set, fall back to `streaming_conninfo`
-        wal_streaming_conninfo = self.wal_streaming_conninfo or self.streaming_conninfo
+        overrides = {}
+        if self.wal_streaming_conninfo is not None:
+            overrides["streaming_conninfo"] = self.wal_streaming_conninfo
+        # If `wal_streaming_conninfo` is not set, fall back to `streaming_conninfo`.
+        wal_streaming_conninfo = overrides.get(
+            "streaming_conninfo", self.streaming_conninfo
+        )
 
         # If `wal_conninfo` is not set, fall back to `wal_streaming_conninfo`. If
         # `wal_streaming_conninfo` is not set, fall back to `conninfo`.
         if self.wal_conninfo is not None:
-            wal_conninfo = self.wal_conninfo
+            overrides["conninfo"] = self.wal_conninfo
         elif self.wal_streaming_conninfo is not None:
-            wal_conninfo = self.wal_streaming_conninfo
-        else:
-            wal_conninfo = self.conninfo
+            overrides["conninfo"] = self.wal_streaming_conninfo
+        wal_conninfo = overrides.get("conninfo", self.conninfo)
+
+        if overrides:
+            message_parts = [
+                f"'{key}' overridden with '{value}'" for key, value in overrides.items()
+            ]
+            # Only display message if its not a check command!
+            if not check_strategy:
+                output.info(
+                    "Configuration overrides applied: %s", " and ".join(message_parts)
+                )
         return wal_streaming_conninfo, wal_conninfo
 
 
@@ -1464,6 +1501,15 @@ class Config(object):
         """
         json_dict = dict(self._global_config)
 
+        # Add global configuration to output of diagnose that are not present in
+        # `_global_config`.
+        json_dict.update(
+            dict(
+                barman_lock_directory=self.barman_lock_directory,
+                lock_directory_cleanup=self.lock_directory_cleanup,
+                config_changes_queue=self.config_changes_queue,
+            )
+        )
         if with_source:
             for option, value in json_dict.items():
                 json_dict[option] = {
@@ -1489,7 +1535,7 @@ class Config(object):
             return
 
         if not os.path.isdir(os.path.expanduser(config_files_directory)):
-            _logger.warn(
+            _logger.warning(
                 'Ignoring the "configuration_files_directory" option as "%s" '
                 "is not a directory",
                 config_files_directory,
@@ -1517,10 +1563,12 @@ class Config(object):
                     raise SystemExit("FATAL: %s" % msg)
             else:
                 # Add an warning message that a file has been discarded
-                _logger.warn("Discarding configuration file: %s (not a file)", filename)
+                _logger.warning(
+                    "Discarding configuration file: %s (not a file)", filename
+                )
         else:
             # Add an warning message that a file has been discarded
-            _logger.warn("Discarding configuration file: %s (not found)", filename)
+            _logger.warning("Discarding configuration file: %s (not found)", filename)
 
     def _is_model(self, name):
         """
