@@ -26,7 +26,7 @@ import re
 import dateutil.parser
 import dateutil.tz
 
-from barman import xlog
+from barman import output, xlog
 from barman.cloud_providers import snapshots_info_from_dict
 from barman.exceptions import BackupInfoBadInitialisation
 from barman.utils import fsync_dir
@@ -336,7 +336,16 @@ class FieldListFile(object):
                     value = None
                 elif isinstance(field, Field) and callable(field.from_str):
                     value = field.from_str(value)
-                setattr(self, name, value)
+                try:
+                    setattr(self, name, value)
+                except AttributeError:
+                    output.error(
+                        "Unsupported field '%s' found in backup metadata. This "
+                        "indicates the backup was created with a newer version of "
+                        "Barman. Please upgrade to a compatible or newer version.",
+                        name,
+                    )
+                    output.close_and_exit()
 
     def items(self):
         """
@@ -575,6 +584,15 @@ class BackupInfo(FieldListFile):
     encryption = Field("encryption")
 
     __slots__ = "backup_id", "backup_version"
+    #: "backup_version": Indicates the internal backup directory layout version.
+    #:
+    #: * ``1`` – "Legacy" layout with ``pgdata``.
+    #: * ``2`` – New layout (introduced in commit f4729f3...), where ``pgdata`` was
+    #:           renamed to ``data``. Tablespaces are stored alongside ``data`` and
+    #:           copied via dedicated rsync commands.
+    #:
+    #: Barman detects the "backup_version" based on directory structure:
+    #: ``pgdata`` -> 1, ``data`` -> 2 (default).
 
     _hide_if_null = ("backup_name", "snapshots_info")
 
@@ -877,14 +895,28 @@ class LocalBackupInfo(BackupInfo):
                 e,
             )
 
-    def get_list_of_files(self, target):
+    def get_directory_entries(self, target, empty_dirs=False):
         """
-        Get the list of files for the current backup
+        Get the list of files and optionally empty directories for the current backup.
+
+        :param str target: The type of entry to target. One among:
+
+            * ``data``: all files in the base backup directory.
+            * ``wal``: all WAL files until the next backup.
+            * ``full``: same as ``data`` plus ``wal``.
+            * ``standalone``: same as ``data`` plus the WAL files required for the
+                consistency of this backup.
+
+        :param bool empty_dirs: Whether to include empty directories in the list.
+            Defaults to ``False``.
+        :yield str: The path to a file or an empty directory.
         """
         # Walk down the base backup directory
         if target in ("data", "standalone", "full"):
-            for root, _, files in os.walk(self.get_basebackup_directory()):
+            for root, dirs, files in os.walk(self.get_basebackup_directory()):
                 files.sort()
+                if empty_dirs and len(dirs + files) == 0:
+                    yield root
                 for f in files:
                     yield os.path.join(root, f)
         if target in "standalone":
@@ -913,12 +945,20 @@ class LocalBackupInfo(BackupInfo):
                 return match.group(1)
             return os.path.basename(os.path.dirname(self.filename))
 
+    def get_base_directory(self):
+        """
+        Retrieve the base directory for this backup
+
+        :return str: the base directory for this backup
+        """
+        return self.config.basebackups_directory
+
     def get_basebackup_directory(self):
         """
         Get the default filename for the backup.info file based on
         backup ID and server directory for base backups
         """
-        return os.path.join(self.config.basebackups_directory, self.backup_id)
+        return os.path.join(self.get_base_directory(), self.backup_id)
 
     def get_data_directory(self, tablespace_oid=None):
         """
@@ -1167,44 +1207,6 @@ class LocalBackupInfo(BackupInfo):
         return False
 
 
-class SyntheticBackupInfo(LocalBackupInfo):
-    def __init__(
-        self, server, base_directory, backup_id=None, info_file=None, **kwargs
-    ):
-        """
-        Stores meta information about a single synthetic backup.
-
-        .. note::
-            A synthetic backup is a base backup which was artificially created
-            through ``pg_combinebackup``. A synthetic backup is not part of
-            the Barman backup catalog, and only exists so we are able to
-            recover a backup created by ``pg_combinebackup`` utility, as
-            almost all functions and methods require a backup info object.
-
-        The only difference from this class to its parent :class:`LocalBackupInfo`
-        is that it accepts a custom base directory for the backup as synthetic
-        backups are expected to live on directories other than the default
-        ``<server_name>/base`` path.
-
-        :param barman.server.Server server: the server that owns the
-            synthetic backup
-        :param str base_directory: the root directory where this synthetic
-            backup resides, essentially an override to the
-            ``server.config.basebackups_directory`` configuration.
-        :param str|None backup_id: the backup id of this backup
-        :param None|str|TextIO info_file: path or file descriptor of an existing
-            synthetic ``backup.info`` file
-        """
-        self.base_directory = base_directory
-        super(SyntheticBackupInfo, self).__init__(
-            server, info_file, backup_id, **kwargs
-        )
-
-    def get_basebackup_directory(self):
-        """Get the backup directory based on its base directory"""
-        return os.path.join(self.base_directory, self.backup_id)
-
-
 class VolatileBackupInfo(LocalBackupInfo):
     def __init__(
         self, server, base_directory, backup_id=None, info_file=None, **kwargs
@@ -1231,12 +1233,20 @@ class VolatileBackupInfo(LocalBackupInfo):
             option.
         :param str|None backup_id: The backup id of the backup.
         :param None|str|TextIO info_file: The path to an existing ``backup.info`` file,
-        or
-            a file-like object from which to read the backup information.
+            or a file-like object from which to read the backup information.
         """
         self.base_directory = base_directory
+        self.parent_instance = None
         super(VolatileBackupInfo, self).__init__(server, info_file, backup_id, **kwargs)
 
+    def get_base_directory(self):
+        """
+        Retrieve the base directory for this backup.
+
+        :return str: The base directory for this backup.
+        """
+        return self.base_directory
+
     def get_basebackup_directory(self):
         """
         Retrieve the full path to the base backup directory for this backup.
@@ -1250,10 +1260,42 @@ class VolatileBackupInfo(LocalBackupInfo):
         """
         return os.path.join(self.base_directory, self.backup_id)
 
-    def save(self, *args, **kwargs):
+    def walk_to_root(self, return_self=True):
         """
-        Override the save method to prevent its usage.
+        Walk through all the parent backups of the current backup.
 
-        :raises NotImplementedError: This method is not implemented.
+        .. note::
+            This method is similar to the one in :class:`LocalBackupInfo`, except that
+            it traverses the tree based on the in-memory :attr:`parent_instance`
+            attribute.
+
+        :param bool return_self: Whether to return the current backup.
+            Default to ``True``.
+        :yield: a generator of :class:`LocalBackupInfo` objects for each parent backup.
         """
-        raise NotImplementedError("The save method is not implemented.")
+        if return_self:
+            yield self
+        backup_info = self.parent_instance
+        while backup_info:
+            yield backup_info
+            backup_info = backup_info.parent_instance
+
+    def save(self, filename=None, file_object=None):
+        """
+        Serialize the object to the specified file object
+
+        .. note::
+            This method overrides the :meth:`save` method of :class:`LocalBackupInfo` to
+            prevent saving to a file. It raises a :exec:`ValueError` if no file-like object
+            is provided, as this class is intended for in-memory operations only.
+            However, we still maintain the same interface to allow :class:`LocalBackupInfo`
+            objects to be used interchangeably in some contexts.
+
+        :param str filename: path of the file to write
+        :param file file_object: a file like object to write in
+        :param str filename: the file to write
+        :raises: ValueError
+        """
+        if not file_object:
+            raise ValueError("VolatileBackupInfo does not support saving to a file")
+        super(LocalBackupInfo, self).save(filename=filename, file_object=file_object)
