@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2023
+# © Copyright EnterpriseDB UK Limited 2011-2025
 #
 # This file is part of Barman.
 #
@@ -17,8 +17,6 @@
 # along with Barman.  If not, see <http://www.gnu.org/licenses/>
 
 import collections
-import datetime
-import errno
 import filecmp
 import logging
 import os
@@ -45,7 +43,7 @@ _logger = logging.getLogger(__name__)
 
 
 class WalArchiverQueue(list):
-    def __init__(self, items, errors=None, skip=None, batch_size=0):
+    def __init__(self, items, errors=None, skip=None, batch_size=0, total_size=0):
         """
         A WalArchiverQueue is a list of WalFileInfo which has two extra
         attribute list:
@@ -58,10 +56,21 @@ class WalArchiverQueue(list):
         number of WAL files that are processed in a single
         run of the archive-wal command.
 
+        .. note::
+            This class was originally designed to hold all the WAL files pending to be
+            archived, and to process up to a certain number of files (the batch size).
+            However, it is now used to hold only the files that are being processed in a
+            single run of the archive-wal command. That change was made to avoid the
+            overhead of having to create lots of WalFileInfo objects, even if only a
+            subset of them would be used by the archiver in a given batch. We might want
+            to rework or remove this class in the future, as it doesn't seem to add much
+            value to the archiving process anymore.
+
         :param items: iterable from which initialize the list
-        :param batch_size: size of the current batch run (0=unlimited)
         :param errors: an optional list of unrecognized files
         :param skip: an optional list of skipped files
+        :param batch_size: size of the current batch run (0=unlimited)
+        :param total_size: the total number of WAL files available for archiving.
         """
         super(WalArchiverQueue, self).__init__(items)
         self.skip = []
@@ -76,30 +85,8 @@ class WalArchiverQueue(list):
         else:
             self.batch_size = 0
 
-    @property
-    def size(self):
-        """
-        Number of valid WAL segments waiting to be processed (in total)
-
-        :return int: total number of valid WAL files
-        """
-        return len(self)
-
-    @property
-    def run_size(self):
-        """
-        Number of valid WAL files to be processed in this run - takes
-        in consideration the batch size
-
-        :return int: number of valid WAL files for this batch run
-        """
-        # In case a batch size has been explicitly specified
-        # (i.e. batch_size > 0), returns the minimum number between
-        # batch size and the queue size. Otherwise, simply
-        # returns the total queue size (unlimited batch size).
-        if self.batch_size > 0:
-            return min(self.size, self.batch_size)
-        return self.size
+        self.total_size = total_size
+        self.run_size = len(self)
 
 
 class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
@@ -138,7 +125,7 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
         :param boolean verbose: Flag for verbose output
         """
         compressor = self.backup_manager.compression_manager.get_default_compressor()
-        stamp = datetime.datetime.utcnow().strftime("%Y%m%dT%H%M%SZ")
+        encryption = self.backup_manager.encryption_manager.get_encryption()
         processed = 0
         header = "Processing xlog segments from %s for %s" % (
             self.name,
@@ -149,13 +136,13 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
         batch = self.get_next_batch()
 
         # Analyse the batch and properly log the information
-        if batch.size:
-            if batch.size > batch.run_size:
+        if batch.run_size:
+            if batch.total_size > batch.run_size:
                 # Batch mode enabled
                 _logger.info(
                     "Found %s xlog segments from %s for %s."
                     " Archive a batch of %s segments in this run.",
-                    batch.size,
+                    batch.total_size,
                     self.name,
                     self.config.name,
                     batch.run_size,
@@ -166,7 +153,7 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
                 _logger.info(
                     "Found %s xlog segments from %s for %s."
                     " Archive all segments in one run.",
-                    batch.size,
+                    batch.total_size,
                     self.name,
                     self.config.name,
                 )
@@ -185,16 +172,6 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
             if not processed and not verbose:
                 output.info(header, log=False)
 
-            # Exit when archive batch size is reached
-            if processed >= batch.run_size:
-                _logger.debug(
-                    "Batch size reached (%s) - Exit %s process for %s",
-                    batch.batch_size,
-                    self.name,
-                    self.config.name,
-                )
-                break
-
             processed += 1
 
             # Report to the user the WAL file we are archiving
@@ -209,25 +186,21 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
             )
             # Archive the WAL file
             try:
-                self.archive_wal(compressor, wal_info)
+                self.archive_wal(compressor, encryption, wal_info)
             except MatchingDuplicateWalFile:
                 # We already have this file. Simply unlink the file.
                 os.unlink(wal_info.orig_filename)
                 continue
             except DuplicateWalFile:
+                self.server.move_wal_file_to_errors_directory(
+                    wal_info.orig_filename, wal_info.name, "duplicate"
+                )
                 output.info(
                     "\tError: %s is already present in server %s. "
                     "File moved to errors directory.",
                     wal_info.name,
                     self.config.name,
                 )
-                error_dst = os.path.join(
-                    self.config.errors_directory,
-                    "%s.%s.duplicate" % (wal_info.name, stamp),
-                )
-                # TODO: cover corner case of duplication (unlikely,
-                # but theoretically possible)
-                shutil.move(wal_info.orig_filename, error_dst)
                 continue
             except AbortedRetryHookScript as e:
                 _logger.warning(
@@ -238,10 +211,18 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
                 return
 
         if processed:
+            if batch.total_size > batch.run_size:
+                _logger.debug(
+                    "Batch size reached (%s) - Exit %s process for %s",
+                    batch.batch_size,
+                    self.name,
+                    self.config.name,
+                )
+
             _logger.debug(
                 "Archived %s out of %s xlog segments from %s for %s",
                 processed,
-                batch.size,
+                batch.total_size,
                 self.name,
                 self.config.name,
             )
@@ -274,23 +255,18 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
                     self.name,
                     basename,
                 )
-                error_dst = os.path.join(
-                    self.config.errors_directory, "%s.%s.unknown" % (basename, stamp)
+                self.server.move_wal_file_to_errors_directory(
+                    error, basename, "unknown"
                 )
-                try:
-                    shutil.move(error, error_dst)
-                except IOError as e:
-                    if e.errno == errno.ENOENT:
-                        _logger.warning("%s not found" % error)
 
-    def archive_wal(self, compressor, wal_info):
+    def archive_wal(self, compressor, encryption, wal_info):
         """
         Archive a WAL segment and update the wal_info object
 
         :param compressor: the compressor for the file (if any)
+        :param None|Encryption encryption: the encryptor for the file (if any)
         :param WalFileInfo wal_info: the WAL file is being processed
         """
-
         src_file = wal_info.orig_filename
         src_dir = os.path.dirname(src_file)
         dst_file = wal_info.fullpath(self.server)
@@ -315,15 +291,26 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
 
             # Check if destination already exists
             if os.path.exists(dst_file):
+                dst_info = self.backup_manager.get_wal_file_info(dst_file)
                 src_uncompressed = src_file
                 dst_uncompressed = dst_file
-                dst_info = comp_manager.get_wal_file_info(dst_file)
                 try:
+                    # If the existing destination file is already encrypted, it can't be
+                    # decrypted or uncompressed to perform any of the later comparisons
+                    # (because we cannot assume the encryption passphrase is always
+                    # available in the configuration).
+                    if dst_info.encryption:
+                        raise DuplicateWalFile(wal_info)
+                    # If the existing file is already compressed, decompress it to a
+                    # <dst_wal_path>.uncompressed file
                     if dst_info.compression is not None:
                         dst_uncompressed = dst_file + ".uncompressed"
                         comp_manager.get_compressor(dst_info.compression).decompress(
                             dst_file, dst_uncompressed
                         )
+                    # If the source file is already compressed (because the user
+                    # compressed it manually with a script in the archive_command),
+                    # then decompress it to a <src_wal_path>.uncompressed file
                     if wal_info.compression:
                         src_uncompressed = src_file + ".uncompressed"
                         comp_manager.get_compressor(wal_info.compression).decompress(
@@ -344,36 +331,52 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
                         os.unlink(dst_uncompressed)
 
             mkpath(dst_dir)
-            # Compress the file only if not already compressed
+
+            # List of intermediate files that will need to be removed after the archival
+            files_to_remove = []
+            # The current working file being touched
+            current_file = src_file
+            # If the bits of the file has changed e.g. due to compression or encryption
+            content_changed = False
+            # Compress the file if not already compressed
             if compressor and not wal_info.compression:
                 compressor.compress(src_file, tmp_file)
+                files_to_remove.append(current_file)
+                current_file = tmp_file
+                content_changed = True
+                wal_info.compression = compressor.compression
+            # Encrypt the file
+            if encryption:
+                encrypted_file = encryption.encrypt(current_file, dst_dir)
+                files_to_remove.append(current_file)
+                current_file = encrypted_file
+                wal_info.encryption = encryption.NAME
+                content_changed = True
 
             # Perform the real filesystem operation with the xlogdb lock taken.
             # This makes the operation atomic from the xlogdb file POV
             with self.server.xlogdb("a") as fxlogdb:
-                if compressor and not wal_info.compression:
-                    shutil.copystat(src_file, tmp_file)
-                    os.rename(tmp_file, dst_file)
-                    os.unlink(src_file)
-                    # Update wal_info
-                    stat = os.stat(dst_file)
+                # If the content has changed, it means the file was either compressed
+                # or encrypted or both. In this case, we need to update its metadata
+                if content_changed:
+                    shutil.copystat(src_file, current_file)
+                    stat = os.stat(current_file)
                     wal_info.size = stat.st_size
-                    wal_info.compression = compressor.compression
-                else:
-                    # Try to atomically rename the file. If successful,
-                    # the renaming will be an atomic operation
-                    # (this is a POSIX requirement).
-                    try:
-                        os.rename(src_file, dst_file)
-                    except OSError:
-                        # Source and destination are probably on different
-                        # filesystems
-                        shutil.copy2(src_file, tmp_file)
-                        os.rename(tmp_file, dst_file)
-                        os.unlink(src_file)
+
+                # Try to atomically rename the file. If successful, the renaming will
+                # be an atomic operation (this is a POSIX requirement).
+                try:
+                    os.rename(current_file, dst_file)
+                except OSError:
+                    # Source and destination are probably on different filesystems
+                    shutil.copy2(current_file, tmp_file)
+                    os.rename(tmp_file, dst_file)
+                finally:
+                    for file in files_to_remove:
+                        os.unlink(file)
+
                 # At this point the original file has been removed
                 wal_info.orig_filename = None
-
                 # Execute fsync() on the archived WAL file
                 fsync_file(dst_file)
                 # Execute fsync() on the archived WAL containing directory
@@ -518,6 +521,12 @@ class FileWalArchiver(WalArchiver):
         # verify that a backup has all the WALs required for the restore.
         file_names = glob(os.path.join(self.config.incoming_wals_directory, "*"))
         file_names.sort()
+        total_size = len(file_names)
+        # If batch size is set, limit the number of files to the batch size. The idea
+        # is to avoid the overhead of creating several unused WalFileInfo objects. See
+        # the note in the WalArchiverQueue class.
+        if batch_size > 0:
+            file_names = file_names[:batch_size]
 
         # Process anything that looks like a valid WAL file. Anything
         # else is treated like an error/anomaly
@@ -534,10 +543,23 @@ class FileWalArchiver(WalArchiver):
 
         # Build the list of WalFileInfo
         wal_files = [
-            WalFileInfo.from_file(f, self.backup_manager.compression_manager)
+            WalFileInfo.from_file(
+                filename=f,
+                compression_manager=self.backup_manager.compression_manager,
+                unidentified_compression=None,
+                encryption_manager=self.backup_manager.encryption_manager,
+                # We don't try to guess if the WAL is encrypted here. If the user sets
+                # an archive_command which encrypts the WAL file, it's up to the user to
+                # decrypt them later, and Barman won't do anything about it. We still
+                # attempt to guess compression, though, because that's been the behavior
+                # of Barman for a long time.
+                encryption=None,
+            )
             for f in files
         ]
-        return WalArchiverQueue(wal_files, batch_size=batch_size, errors=errors)
+        return WalArchiverQueue(
+            wal_files, batch_size=batch_size, errors=errors, total_size=total_size
+        )
 
     def check(self, check_strategy):
         """
@@ -942,6 +964,12 @@ class StreamingWalArchiver(WalArchiver):
         # verify that a backup has all the WALs required for the restore.
         file_names = glob(os.path.join(self.config.streaming_wals_directory, "*"))
         file_names.sort()
+        total_size = len(file_names)
+        # If batch size is set, limit the number of files to the batch size. The idea
+        # is to avoid the overhead of creating several unused WalFileInfo objects. See
+        # the note in the WalArchiverQueue class.
+        if batch_size > 0:
+            file_names = file_names[:batch_size]
 
         # Process anything that looks like a valid WAL file,
         # including partial ones and history files.
@@ -981,9 +1009,26 @@ class StreamingWalArchiver(WalArchiver):
             skip.append(files.pop())
 
         # Build the list of WalFileInfo
-        wal_files = [WalFileInfo.from_file(f, compression=None) for f in files]
+        wal_files = [
+            WalFileInfo.from_file(
+                filename=f,
+                compression_manager=self.backup_manager.compression_manager,
+                encryption_manager=self.backup_manager.encryption_manager,
+                unidentified_compression=None,
+                # WAL files received through pg_receivewal are surely not encrypted nor
+                # compressed, so we avoid the overhead of trying to guess such
+                # algorithms.
+                compression=None,
+                encryption=None,
+            )
+            for f in files
+        ]
         return WalArchiverQueue(
-            wal_files, batch_size=batch_size, errors=errors, skip=skip
+            wal_files,
+            batch_size=batch_size,
+            errors=errors,
+            skip=skip,
+            total_size=total_size,
         )
 
     def check(self, check_strategy):
